require 'raykit'

desc "build dotnet solutions"
task :build do
    puts Rainbow(':build').blue.bright
    Dir.glob('**/*.sln').each{|sln|
        PROJECT.run("dotnet build #{sln} --configuration Release")
    }

    Dir.glob('**/*.csproj').each{|csproj|
        text=IO.read(csproj)
        if(text.include?('<GeneratePackageOnBuild>true</GeneratePackageOnBuild>'))
            PROJECT.run("dotnet pack #{csproj} --configuration Release")
        end
        if(text.include?('<OutputType>Exe</OutputType>') ||
           text.include?('<OutputType>WinExe</OutputType>'))
           if(text.include?('<TargetFramework>net5.0-windows</TargetFramework>')) 
                PROJECT.run("dotnet publish #{csproj} -c Release -f net5.0-windows -r win-x64 /p:PublishSingleFile=true /p:UseAppHost=true")
           else
                if(text.include?('<TargetFramework>net5.0</TargetFramework>')) 
                    PROJECT.run("dotnet publish #{csproj} -c Release -f net5.0 -r win-x64 /p:PublishSingleFile=true /p:UseAppHost=true")
                else
                    PROJECT.run("dotnet publish #{csproj} -c Release -f netcoreapp3.1 -r win-x64 /p:PublishSingleFile=true /p:UseAppHost=true")
                end
           end
        end
    }
end

desc "test dotnet projects"
task :test => [:build] do
    puts Rainbow(':test').blue.bright
    Dir.glob('**/*.Test.csproj').each{|csproj|
        PROJECT.run("dotnet test #{csproj} --configuration Release")
    }
end

desc "publish local nuget packages"
task :publish_local => [:build] do
    nuget=PROJECT.get_dev_dir('nuget')
    local_package="#{nuget}/#{PROJECT.name}.#{PROJECT.version}.nupkg"
    if(!File.exist?(local_package))
        package="#{PROJECT.name}/bin/Release/#{PROJECT.name}.#{PROJECT.version}.nupkg"
        PROJECT.run([
            #"dotnet build #{PROJECT.name}.sln --configuration Release",
            #"dotnet pack #{PROJECT.name}.sln -c Release",
            "raykit copy #{package} #{nuget}/#{PROJECT.name}.#{PROJECT.version}.nupkg"
        ])
    end
end

desc "publish to NuGet.org"
task :publish_to_nuget => [:build] do
    if(!ENV['NUGET_KEY'].nil?)
        SECRETS['NUGET_KEY'] = ENV['NUGET_KEY']
        Dir.glob('**/*.nupkg'){|nupkg|
            if(nupkg.include?('Release'))
                PROJECT.run("dotnet nuget push #{nupkg} --skip-duplicate --api-key #{SECRETS['NUGET_KEY']} --source https://api.nuget.org/v3/index.json")
            end
        }   
    end
end

desc "delete nuget package from NuGet.org"
task :delete_from_nuget do
    if(!ENV['NUGET_KEY'].nil?)
        SECRETS['NUGET_KEY'] = ENV['NUGET_KEY']
        Dir.glob('**/*.nupkg'){|nupkg|
            if(nupkg.include?('Release'))
                match = nupkg.match(/([\w.]+[a-zA-Z]).(\d[\d.]+).nupkg/)
                if(!match.nil? && match.captures.length > 1)
                    name=match.captures[0]
                    version=match.captures[1]
                    PROJECT.run("dotnet nuget delete #{name} #{version} --non-interactive --api-key #{SECRETS['NUGET_KEY']} --source https://api.nuget.org/v3/index.json")
                end
            end
        }   
    end
end

#task :integrate => [:test]

desc 'bump the version'
task :bump do
    puts Rainbow(':bump').blue.bright
    old_version = Raykit::Version.detect(PROJECT.name)
    new_version = Raykit::Version.bump_file("#{PROJECT.name}/#{PROJECT.name}.csproj")
    puts "        bumped version from #{old_version} to #{new_version}"
    #puts "        perform rake :integrate to integrate this change."
    puts ''
    if(PROJECT.outstanding_commit?)
        if(Rake::Task.task_defined?("integrate"))
            Rake::Task["integrate"].invoke
        end 
    end
end

desc 'update package versions'
task :update do
    puts Rainbow(':update').blue.bright
    Dir.glob("**/*.csproj").each{|csproj|
        Raykit::DotNet.get_package_names(csproj).each{|package|
            PROJECT.run("dotnet add #{csproj} package #{package}")
        }
    }
    if(PROJECT.outstanding_commit?)
        if(Rake::Task.task_defined?("integrate"))
            Rake::Task["integrate"].invoke
        end 
    end
end

task :default do
    PROJECT.summary
end

desc 'run roslynator fix on dotnet solutions'
task :fix do
    analyzer_dir="#{Raykit::Environment::local_application_data}/Microsoft/VisualStudio"
    puts Rainbow(':fix').blue.bright
    Dir.glob('**/*.sln').each{|sln|
        PROJECT.run("roslynator fix #{sln} --analyzer-assemblies #{analyzer_dir} --msbuild-path \"#{Raykit::MsBuild::msbuild_path}\"")
    }
end

def copy_files(source_dir,glob_pattern,target_dir,exclude_pattern)
    count = 0
    Dir.chdir(source_dir) do
        Dir.glob(glob_pattern).each{|file|
            if(!file.include?(exclude_pattern))
                target_file = target_dir + "/" + file
                target_parent = File.dirname(target_file)
                FileUtils.mkdir_p(target_parent) if(!Dir.exist?(target_parent))
                FileUtils.copy(file,target_file)
                count = count + 1
            end
        }
    end
    puts "        copied #{count} files to " + target_dir
end

# https://docs.microsoft.com/en-us/visualstudio/code-quality/how-to-generate-code-metrics-data?view=vs-2019
def compute_code_metrics
	metrics_file='artifacts/Code-Metrics.xml'
	FileUtils.mkdir('artifacts') if(!Dir.exists?('artifacts'))
	File.delete('artifacts/Code-Metrics.xml') if(File.exists?('artifacts/Code-Metrics.xml'))
	PROJECT.run("metrics /s:#{PROJECT.name}.sln /o:artifacts/Code-Metrics.xml")
	if(!File.exist?(metrics_file))
		raise "#{metrics_file} does not exist"
	end
end

desc 'initialize a dotnet library'
task :init_lib do
    Raykit::DotNet::initialize_csharp_lib(PROJECT.name)
end